'use strict';
function GenerateGameLink(teamId, tournamentId){
	return "/teams/" + teamId + (tournamentId ? "/tournaments/" + tournamentId : "") + "/games";
}
function GenerateDemoTeamData(){
	var demoTeamId = GetTime();
	var demoOpponentId = GetTime()+1;

	var demoData = {
		id: demoTeamId,
		name: "Discemos",
		size: 7,
		created: GetTime(),
		modified: GetTime(),
		synced: null,
		opponents: [{
			id: demoOpponentId,
			teamId: demoTeamId,
			name: "Runchers",
			created: GetTime(),
			modified: GetTime(),
			synced: null
		}],
		players: [
			{id: GetTime()+4, firstName: "", lastName: "", name: "Mat", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null},
			{id: GetTime()+5, firstName: "", lastName: "", name: "Karl", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null},
			{id: GetTime()+6, firstName: "", lastName: "", name: "Alex", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null},
			{id: GetTime()+7, firstName: "", lastName: "", name: "Colin", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null},
			{id: GetTime()+8, firstName: "", lastName: "", name: "Carly", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null},
			{id: GetTime()+9, firstName: "", lastName: "", name: "Anne", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null},
			{id: GetTime()+10, firstName: "", lastName: "", name: "Liz", line: null, type: null, created: GetTime(),modified: GetTime(), synced: null}
		]
	};
	return {team: demoData, teamId: demoTeamId, opponentId: demoOpponentId};
}
function GetTime(){
	return Math.round(GetMicroTime() / 1000);
}
function GetMicroTime(){
	return (new Date()).getTime();
}
function GenerateRandomNumber(){
	return GetTime();
}
function ArrayIndexOf(array, key, search){
	for(var x = 0, len = array.length; x < len; x++){
		if(array[x][key] == search){
			return x;
		}
	}
	return -1;
}
function Clone(object){
	var newObj = (object instanceof Array) ? [] : {};
	for (var i in object) {
		if(object.hasOwnProperty(i)){
			newObj[i] = (object[i] && typeof object[i] == "object") ? Clone(object[i]) : object[i];
		}
	}
	return newObj;
}
function ArrowHead(x0, y0, angle, length, widthAngle){
	return {
		x0: x0,
		y0: y0,
		x1: Math.round(x0 + length * Math.cos((angle - Math.PI) + widthAngle)),
		y1: Math.round(y0 - length * Math.sin((angle - Math.PI) + widthAngle)),
		x2: Math.round(x0 - length * Math.cos(angle - widthAngle)),
		y2: Math.round(y0 + length * Math.sin(angle - widthAngle))
	}
}
function GetGradient(gradient){
	var gradientCanvas = document.createElement("canvas");
	var gradientCanvasContext = gradientCanvas.getContext("2d");

	gradientCanvas.width = 1;
	gradientCanvas.height = 256;

	var grad = gradientCanvasContext.createLinearGradient(0, 0, 1, 256);

	for (var x = 0, len = gradient.length; x < len; x++) {
		grad.addColorStop(gradient[x].step, gradient[x].color);
	}
	gradientCanvasContext.fillStyle = grad;
	gradientCanvasContext.fillRect(0, 0, 1, 256);

	return gradientCanvasContext.getImageData(0, 0, 1, 256).data;
}
function CheckParent(element, className){
	if(angular.element(element).hasClass(className)){
		return element;
	}
	return element.parentNode ? CheckParent(element.parentNode, className) : false;
}
function GetElementOffset(elem, type){
	var offset = 0;
	do{ offset += !isNaN (elem[type]) ? elem[type] : 0; }
	while( elem = elem.offsetParent );
	return offset;
}
function Last(array, back){
	return array[array.length - (back ? back : 1)];
}
function StopEvent(event){
	if(event){
		event.stopPropagation();
		event.preventDefault();
	}
}
function KeyToText(object){
	var text = {};
	for (var key in object) {
		if(object.hasOwnProperty(key)){
			text[object[key]] = key;
		}
	}
	return text;
}
function MergeObjects(object, merger){
	for(var key in merger){
		if(merger.hasOwnProperty(key)){
			var parsed = parseInt(merger[key], 10);
			if(merger[key] == parsed){
				merger[key] = parsed;
			}
			object[key] = merger[key];
		}
	}
}
function IsNewer(v1, v2){
	var v1parts = v1.split('.');
	var v2parts = v2.split('.');

	for (var i = 0, len = v1parts.length; i < len; ++i) {
		if (v2parts.length == i) {
			return false;
		}
		if (v1parts[i] != v2parts[i]) {
			return v1parts[i] <= v2parts[i];
		}
	}
	return true;
}
function AndroidTouchScrollElement(element){
	var androidSplit = navigator.userAgent.toLowerCase().split("android ");
	if(androidSplit.length < 2 || IsNewer('3.0.0', androidSplit[1].split(";")[0])){ return; }

	var scrollStartPosY = 0;
	var scrollStartPosX = 0;

	element.addEventListener("touchstart", function(event) {
		scrollStartPosY = this.scrollTop + event.touches[0].pageY;
		scrollStartPosX = this.scrollLeft + event.touches[0].pageX;
	},false);

	element.addEventListener("touchmove", function(event) {
		if ((this.scrollTop < this.scrollHeight - this.offsetHeight && this.scrollTop + event.touches[0].pageY < scrollStartPosY-5) || (this.scrollTop != 0 && this.scrollTop + event.touches[0].pageY > scrollStartPosY + 5)){
			event.preventDefault();
		}
		if ((this.scrollLeft < this.scrollWidth - this.offsetWidth && this.scrollLeft + event.touches[0].pageX < scrollStartPosX-5) || (this.scrollLeft != 0 && this.scrollLeft + event.touches[0].pageX > scrollStartPosX + 5)){
			event.preventDefault();
		}
		this.scrollTop = scrollStartPosY - event.touches[0].pageY;
		this.scrollLeft = scrollStartPosX - event.touches[0].pageX;
	},false);
}