'use strict';
var v = GetTime();

if(navigator && navigator.notification && navigator.notification.alert){ window.alert = navigator.notification.alert; }

var actionTypes = {
	blank:0, 				//blank, used to send back if nothing found
	pickUp:1, 				//Used for start of a point/injury/turnover/etc...
	pass:2, 				//A completed pass from one person to another
	point:3, 				//The pass was a point, this will change the last action which was a pass to become a point
	drop:4, 				//The pass should be counted as a drop, this will change the last action which was a pass to become a drop (used both offense/defense)
	throwaway:5, 			//The pass was a throwaway, this will change the last action which was a pass to become a throwaway (used both offense/defense)
	pull:6, 				//The pull, used to track where the pull landed and assign it to a person (offense/defense)
	block:7, 				//The pass was a block, this will change the last action which was a pass to become a block (offense/defense)
	timeout:8, 				//The last action person will be credited with a timeout
	injuryStart:9, 			//Injury called, no person is assigned
	injuryEnd:10, 			//End of the injury
	interception:11, 		//A block and then a pickup (not actually saved)
	callahan:12, 			//A	block, pickup and then point (not actually saved)
	stallDown:13,			//only available after a pickup or a pass
	halfTime:14             //Half time
};
var badOffenseActions = [
	actionTypes.drop,
	actionTypes.throwaway,
	actionTypes.stallDown
];
var badDefenseActions = [
	actionTypes.pickUp,
	actionTypes.pass,
	actionTypes.block,
	actionTypes.point,
	actionTypes.timeout
];
var ignoreActions = [
	actionTypes.blank,
	actionTypes.injuryStart,
	actionTypes.injuryEnd,
	actionTypes.interception,
	actionTypes.callahan,
	actionTypes.halfTime
];
var selectableActions = [
	{ name: "Point",		type: actionTypes.point,		description: "The pass as a point"},
	{ name: "Timeout",		type: actionTypes.timeout,		description: "Timeout was called by the person with the disc"},
	{ name: "Injury",		type: actionTypes.injuryStart,	description: "An injury was called on the field"},
	{ name: "Callahan",		type: actionTypes.callahan,		description: "Pass was caught by the defense for a point"}
];

var strategy = 	  { offense:0, 	defense:1 };
var fieldSide =   { left:0, 		right:1 };

var passType = { good: 1, bad: 2, goal: 3};

var actionTypesText = KeyToText(actionTypes);
var strategyText = KeyToText(strategy);
var fieldSideText = KeyToText(fieldSide);

var heatMapOptions = {
	order: 1, name: "Offense", available: {}
};

for(var key in actionTypes){
	if(actionTypes.hasOwnProperty(key)){
		if(ignoreActions.indexOf(actionTypes[key]) === -1){
			heatMapOptions.available[actionTypes[key]] = {name: actionTypesText[actionTypes[key]], value: true};
		}
	}
}

var touchstart = 'ontouchstart' in document.documentElement ? 'touchstart' : 'mousedown';
var touchend = 'ontouchend' in document.documentElement ? 'touchend' : 'mouseup';
var touchmove = 'ontouchmove' in document.documentElement ? 'touchmove' : 'mousemove';

var contentHeight = 0;
var contentWidth = 0;
var touchControllerHeight = 0;
var contextSpacing = 10;
var fieldPositionPrecision = 10000;
var teamSizes = [4,5,6,7];
var maxStorageSize = 2.5 * 1024 * 1024 - 20000; //20000 for overhead
var currentScroll = 0;

var playerLines = { offense:1, 	defense:2 };
var playerTypes = { handler:1, 	cutter:2 };

var MOUSE_BUTTON_RIGHT = 3;
var MOUSE_BUTTON_MIDDLE = 2;

var isMobile = {
	Android: function() {
		return navigator.userAgent.match(/Android/i) ? true : false;
	},
	BlackBerry: function() {
		return navigator.userAgent.match(/BlackBerry/i) ? true : false;
	},
	iOS: function() {
		return navigator.userAgent.match(/iPhone|iPad|iPod/i) ? true : false;
	},
	Windows: function() {
		return navigator.userAgent.match(/IEMobile/i) ? true : false;
	},
	any: function() {
		return (isMobile.Android() || isMobile.BlackBerry() || isMobile.iOS() || isMobile.Windows());
	}
};