'use strict';
angular.module('game', [])
		.directive('actionMarker', function(){
			 return {
			 restrict: 'C',
				link: function(scope, element){
					element.css({
						left: (scope.action.x / fieldPositionPrecision * 100) + "%",
						top: (scope.action.y / fieldPositionPrecision * 100) + "%"
					});
				}
			}
		});
function HeatMapController($scope, $location, $routeParams, ST_Storage, ST_Localization){
	PlayGameController($scope, $location, $routeParams, ST_Storage, ST_Localization);
	$scope.HeatMap();
}
function GameStatsController($scope, $location, $routeParams, ST_Storage, ST_Localization){
	PlayGameController($scope, $location, $routeParams, ST_Storage, ST_Localization);
	$scope.GameStats();
}
function PlayGameController($scope, $location, $routeParams, ST_Storage, ST_Localization){
	$scope.game = ST_Storage.getGame($routeParams.gameId);
	$scope.MenuClick = function(){
		$location.path('/teams/' + $routeParams.teamId + '/games');
	};
	if(!$scope.game){ $scope.MenuClick(); }

	AdjustHeight(true);

	window.onresize 			= function(){ AdjustHeight(false, $scope.wind, $scope.field, $scope.heatMap, $scope.discMovement, $scope.playerMovement); };
	window.onorientationchange 	= function(){ AdjustHeight(false, $scope.wind, $scope.field, $scope.heatMap, $scope.discMovement, $scope.playerMovement); };

	$scope.team 				= ST_Storage.getTeam($scope.game.teamId);
	$scope.opponent 			= ST_Storage.getTeamKey($scope.game.teamId, 'opponents', $scope.game.opponentId);

	if(!$scope.team.players || $scope.team.players.length < $scope.team.size){
		$scope.MenuClick();
		alert("You need at least " + $scope.team.size + " players");
		return;
	}

	$scope.actionTypes 			= actionTypes;
	$scope.strategy 			= strategy;
	$scope.strategyText 		= strategyText;
	$scope.side 				= fieldSide;
	$scope.heatMapOptions 		= heatMapOptions;
	$scope.selectableActions 	= selectableActions;

	$scope.field 				= new Field(document.getElementById("fieldCanvas"));
	$scope.points 				= new PointTracker($scope.game, $scope.team.players, $scope.team.name, $scope.opponent.name, $scope.field, ST_Localization, ST_Storage);
	$scope.gameStats 			= new Stats($scope.team.players);
	$scope.discMovement 		= new DiscMovement(document.getElementById("discMovementLeft"), document.getElementById("discMovementRight"));
	$scope.playerMovement 		= new PlayerMovement(document.getElementById("playerMovementLeft"), document.getElementById("playerMovementRight"));
	$scope.heatMap 				= new HeatMap(document.getElementById("heatMapLeft"), document.getElementById("heatMapRight"), $scope.heatMaps);
	$scope.wind 				= new Wind(document.getElementById("windCanvas"), $scope, $scope.points.currentPoint? $scope.points.currentPoint.wind : { intensity: 0, direction: 0});
	$scope.unSelectedPlayers	= {};

	$scope.pointsBackwards = function(point){
		return -((point.scoreFor + point.scoreAgainst) - ((Last(point.actions, 1).a === actionTypes.halfTime) ? 0.5 : 0));
	};
	$scope.UndoClick = function(){
		$scope.points.undo();
	};
	$scope.HistoryClick = function(){
		$scope.showHistory = !$scope.showHistory;
		$scope.showPointActions = false;
	};
	$scope.heatMapStrategy = strategy.offense;

	$scope.showPointSelect = true;

	$scope.HeatMap = function(){
		$scope.displayHeatMap = !$scope.displayHeatMap;
		$scope.showPointSelect = !$scope.showPointSelect;
		if($scope.displayHeatMap){
			$scope.updateHeatMap();
		}
		$scope.displayOptions = false;
	};
	$scope.updateHeatMap = function(){
		$scope.heatMap.calculate($scope.game.points, parseInt($scope.heatMapStrategy, 10));
	};

	$scope.GameStats = function(){
		$scope.displayGameStats = !$scope.displayGameStats;
		$scope.showPointSelect = !$scope.showPointSelect;
		if($scope.displayGameStats){
			$scope.gameStats.calculateStats($scope.game.points);
		}
		$scope.displayOptions = false;
	};
	$scope.discMovementStrategy = strategy.offense;
	$scope.DiscMovement = function(){
		$scope.displayDiscMovement = !$scope.displayDiscMovement;
		if($scope.displayDiscMovement){
			$scope.updateDiscMovement();
		}
		$scope.displayOptions = false;
	};
	$scope.updateDiscMovement = function(){
		$scope.discMovement.calc($scope.game.points, parseInt($scope.discMovementStrategy, 10));
	};
	$scope.playerMovementStrategy = strategy.offense;
	$scope.PlayerMovement = function(){
		$scope.displayPlayerMovement = !$scope.displayPlayerMovement;
		$scope.showPointSelect = !$scope.showPointSelect;
		if($scope.displayPlayerMovement){
			$scope.updatePlayerMovement();
		}
		$scope.displayOptions = false;
	};
	$scope.updatePlayerMovement = function(){
		$scope.playerMovement.calc($scope.game.points, parseInt($scope.playerMovementStrategy, 10), $scope.unSelectedPlayers);
	};
	$scope.FieldMarkers = function(){
		$scope.displayFieldMarkers = !$scope.displayFieldMarkers;
		$scope.displayHeatMaps = false;
	};
	$scope.toggleFieldArrow = function(){
		$scope.showFieldArrow = !$scope.showFieldArrow;
	};
	$scope.toggleBrickMarks = function(){
		$scope.displayBricks = !$scope.displayBricks;
		$scope.displayHeatmaps = false;
	};
	$scope.pointActions = function(point){
		$scope.showPointActions = true;
		$scope.actionList = point;
	};
	$scope.OtherActionsClick = function(){
		$scope.displayOtherActions = !$scope.displayOtherActions;
	};
	$scope.OptionsClick = function(){
		$scope.displayOptions = !$scope.displayOptions;
	};
	$scope.RunAction = function(actionType){
		$scope.points.updateAction(actionType);
		$scope.displayOtherActions = false;
	};
	$scope.debug = function(){
		$scope.displayDebug = !$scope.displayDebug;
	};
	$scope.toggleAllHeatMapOptions = function(){
		var foundEmpty = false;
		for(var key in $scope.heatMapOptions.available){
			if($scope.heatMapOptions.available.hasOwnProperty(key)){
				if($scope.heatMapOptions.available[key].value === false){
					foundEmpty = true;
				}
			}
		}
		for(var key in $scope.heatMapOptions.available){
			if($scope.heatMapOptions.available.hasOwnProperty(key)){
				$scope.heatMapOptions.available[key].value = foundEmpty;
			}
		}
		$scope.updateHeatMap();
	};
	/*angular.element(document).bind(touchstart, function(event){ 	hoverButtonShow($scope, event); });
	angular.element(document).bind(touchmove, function(event){ 		hoverButtonShow($scope, event); });*/

	$scope.$on('$destroy', function() {
		angular.element(document.body).unbind(touchstart);
		angular.element(document.body).unbind(touchmove);
	});

	AndroidTouchScrollElement(document.getElementById("pointScrollContainer"));
	AndroidTouchScrollElement(document.getElementById("actionHistoryList"));
	AndroidTouchScrollElement(document.getElementById("playerMovementOptions"));
	AndroidTouchScrollElement(document.getElementById("gameStatOptions"));

	$scope.currentLinePlayers = [];

	$scope.playPoint = function(){
		$scope.points.playPoint($scope.wind.values, parseInt($scope.points.fieldSide, 10), parseInt($scope.points.strategy, 10), $scope.currentLinePlayers);
		$scope.currentLinePlayers = [];
	};
	$scope.loadOffenseLine = function(){
		$scope.currentLinePlayers = $scope.points.getLastLine(strategy.offense);
	};
	$scope.loadDefenseLine = function(){ 	$scope.currentLinePlayers = $scope.points.getLastLine(strategy.defense); };

	$scope.togglePlayer = function(playerId){
		var playerIndex = $scope.currentLinePlayers.indexOf(playerId);
		if(playerIndex === -1){
			if($scope.currentLinePlayers.length < $scope.team.size){
				$scope.currentLinePlayers.push(playerId);
			}
		}
		else{
			$scope.currentLinePlayers.splice(playerIndex, 1);
		}
	};
	$scope.halfTime = function(){
		$scope.points.halfTime();
	};

	$scope.handlersFirst = function(playerId){
		return $scope.points.playerAssoc(playerId).name;//$scope.points.playerAssoc(playerId) ? $scope.points.playerAssoc(playerId).type : 0;
	};
}
function hoverButtonShow($scope, event){
	var hasTouches = !!event.touches;
	var points = hasTouches ? event.touches[0] : event;
	var hoverElement = CheckParent(document.elementFromPoint(points.clientX, points.clientY), "hoverButton");
	$scope.hoverButtonDescription = hoverElement ? hoverElement.innerHTML : "";

	$scope.$apply();
}
function PointSelectController($scope){
	$scope.orderPlayers = function(player){
		return player.line * 10 + player.type;
	};
}
function AdjustHeight(skipCheck, wind, field, heatMap, discMovement, playerMovement){
	if(!skipCheck && contentHeight === document.body.offsetHeight && contentWidth === document.body.offsetWidth){ return }

	contentHeight = document.documentElement.clientHeight;
	contentWidth = document.documentElement.clientWidth;

	var headerHeight = 38;
	var footerHeight = 38;
	var playerAssignHeight = contentWidth <= 640 ? 40 : 80;
	var innerHeight = contentHeight - headerHeight - footerHeight;

	var touchControllerHeight = innerHeight - playerAssignHeight;

	angular.element(document.getElementsByClassName("overlayPopup")).css({ height: contentHeight + "px"	});
	//angular.element(document.getElementById("gameContainer")).css({ 	minHeight: contentHeight + "px" });
	angular.element(document.getElementById("gameContent")).css({ 		paddingTop: headerHeight + "px", 								paddingBottom: playerAssignHeight + footerHeight + "px" });
	angular.element(document.getElementById("fieldCanvas")).attr({ 		width: contentWidth, 											height: touchControllerHeight });
	angular.element(document.getElementById("playContainer")).css({ 	height: touchControllerHeight + "px" });
	angular.element(document.getElementById("touchContainer")).css({ 	height: touchControllerHeight + "px" });
	angular.element(document.querySelectorAll(".list")).css({ 			top: headerHeight + "px" });

	var availableHeight = contentHeight - headerHeight;

	angular.element(document.getElementById("heatMapOptions")).css({ 	height: availableHeight + "px", 	top: headerHeight + "px" });
	angular.element(document.getElementById("playerMovementOptions")).css({ 	height: availableHeight + "px", 	top: headerHeight + "px" });
	angular.element(document.getElementById("gameStatTable")).css({ 	top: headerHeight + "px",			width : contentWidth + "px", 		height: availableHeight + "px" });
	angular.element(document.getElementById("gameStatOptions")).css({ 	height: availableHeight + "px", 	top: headerHeight + "px" });

	var discMovementCanvasHeight = (contentHeight - headerHeight)/2;
	var discMovementCanvasWidth = discMovementCanvasHeight / 50 * 120;

	if(discMovementCanvasWidth > contentWidth){
		discMovementCanvasWidth = contentWidth;
		discMovementCanvasHeight = discMovementCanvasWidth / 120 * 50;
	}

	discMovementCanvasWidth = Math.floor(discMovementCanvasWidth);
	discMovementCanvasHeight = Math.floor(discMovementCanvasHeight);

	angular.element(document.getElementById("discMovementLeft")).css({ 	width: discMovementCanvasWidth + "px", height: discMovementCanvasHeight + "px", left: (contentWidth - discMovementCanvasWidth) / 2 + "px", top: "38px"});
	angular.element(document.getElementById("discMovementRight")).css({ width: discMovementCanvasWidth + "px", height: discMovementCanvasHeight + "px", left: (contentWidth - discMovementCanvasWidth) / 2 + "px", bottom: "0px"});
	angular.element(document.getElementById("discMovementLeft")).attr({ 	width: discMovementCanvasWidth, height: discMovementCanvasHeight});
	angular.element(document.getElementById("discMovementRight")).attr({ 	width: discMovementCanvasWidth, height: discMovementCanvasHeight});


	angular.element(document.getElementById("playerMovementLeft")).css({ 	width: contentWidth + "px", height: (contentHeight - headerHeight)/2 + "px", left: 0 + "px", top: "38px"});
	angular.element(document.getElementById("playerMovementRight")).css({ width: contentWidth + "px", height: (contentHeight - headerHeight)/2 + "px", left: 0 + "px", bottom: "0px"});
	angular.element(document.getElementById("playerMovementLeft")).attr({ 	width: contentWidth, height: (contentHeight - headerHeight)/2});
	angular.element(document.getElementById("playerMovementRight")).attr({ 	width: contentWidth, height: (contentHeight - headerHeight)/2});


	angular.element(document.getElementById("heatMapLeft")).css({ 	width: discMovementCanvasWidth + "px", height: discMovementCanvasHeight + "px", left: (contentWidth - discMovementCanvasWidth) / 2 + "px", top: "38px"});
	angular.element(document.getElementById("heatMapRight")).css({ width: discMovementCanvasWidth + "px", height: discMovementCanvasHeight + "px", left: (contentWidth - discMovementCanvasWidth) / 2 + "px", bottom: "0px"});
	angular.element(document.getElementById("heatMapLeft")).attr({ 	width: discMovementCanvasWidth, height: discMovementCanvasHeight});
	angular.element(document.getElementById("heatMapRight")).attr({ 	width: discMovementCanvasWidth, height: discMovementCanvasHeight});

	if(wind){ wind.reDraw(); }
	if(field){ field.reDraw(); }
	if(heatMap){ heatMap.reDraw(); }
	if(discMovement){ discMovement.reDraw(); }
	if(playerMovement){ playerMovement.reDraw(); }
}