var scripts = document.getElementsByTagName("script");
var timeScriptLocation = scripts[scripts.length-1].src.replace("script.js", "");

var timeSpacing = 30;
var numShown = 4;
var hoursLength = 24;
var minuteIncrements = 5;

angular.module('st_time', [])
	.service('st_serviceTime', function($rootScope, $compile){
			var serviceTimeScope = this;
			var initialized = false;
			var ngTimeElement;
			var currentObject;
			var currentKey;

			serviceTimeScope.init = function(minutes){
				var timeElement = document.createElement("div");
				ngTimeElement = angular.element(timeElement);

				ngTimeElement.addClass("stTimeElement");
				timeElement.setAttribute("id", "st_time");

				document.body.appendChild(timeElement);

				$compile(timeElement)($rootScope);

				$rootScope.initMinutes = minutes;

				initialized = true;
			};
			serviceTimeScope.open = function(object, key){
				var minutes = 0;
				if(!object[key]){
					var date = new Date();
					minutes = date.getHours() * 60 + Math.round(date.getMinutes() / 5) * 5;
				}
				else{
					minutes = object[key];
				}

				if(!initialized){ serviceTimeScope.init(minutes); }
				else{
					angular.element(document.getElementById("st_time").getElementsByClassName("hourSelect")[0].getElementsByClassName('options')[0]).css({
						top: -Math.floor(minutes / 60) * timeSpacing + "px"
					});
					angular.element(document.getElementById("st_time").getElementsByClassName("minuteSelect")[0].getElementsByClassName('options')[0]).css({
						top: -((minutes % 60) / minuteIncrements * timeSpacing) + "px"
					});
				}

				currentObject = object;
				currentKey = key;

				serviceTimeScope.toggleTime();
			};
			serviceTimeScope.toggleTime = function(minutes){
				ngTimeElement.toggleClass("show");
				if(typeof(minutes) !== "undefined"){
					currentObject[currentKey] = minutes;
				}
			};
		})
	.directive('timeButton', function(st_serviceTime){
			return {
				restrict: 'C',
				link: function(scope, element, attributes){
					element.bind('click', function(){
						if(!attributes.updateObject || !attributes.updateKey|| !scope[attributes.updateObject]){//} || !scope[attributes.updateObject] || !scope[attributes.updateObject][attributes.updateKey]){
							alert("You must include an object and a key to update for the time");
						}
						else{
							st_serviceTime.open(scope[attributes.updateObject], attributes.updateKey);
						}
					})
				}
			};
		})
	.directive('stTimeElement', function(){
			return {
				restrict: 'C',
				controller: TimeController,
				scope: true,
				templateUrl: timeScriptLocation + "index.html?"  + v
			};
		})
	.filter('stRoundDate', function(){
		return function(input){
			return Math.floor(input);
		};
	})
	.filter('stFormatTime', function(){
		return function(input){
			var hours = Math.floor(input / 60);
			var minutes = Math.floor(input % 60);
			return (hours < 10 ? "0" + hours : hours) + ":" + (minutes < 10 ? "0" + minutes : minutes);
		};
	});

function TimeController($scope, st_serviceTime){
	$scope.serviceTime = st_serviceTime;
	$scope.hours = [];
	$scope.hours.length = hoursLength;
	$scope.minuteIncrements = minuteIncrements;
	$scope.minutes = [];
	$scope.minutes.length = 60 / $scope.minuteIncrements;

	angular.element(document.getElementById("st_time").getElementsByClassName("hourSelect")[0].getElementsByClassName('options')[0]).css({
		top: -Math.floor($scope.initMinutes / 60) * timeSpacing + "px"
	});
	angular.element(document.getElementById("st_time").getElementsByClassName("minuteSelect")[0].getElementsByClassName('options')[0]).css({
		top: -Math.floor(($scope.initMinutes % 60) / minuteIncrements * timeSpacing) + "px"
	});

	var scrollElements = angular.element(document.getElementById("st_time").getElementsByClassName("dummy"));
	var hasTouches;
	var startEvent;
	var currentPosition = 0;
	var track = false;
	var newPosition = 0;
	var currentElement;

	scrollElements.bind(touchstart, function(event){
		currentElement = document.getElementById("st_time").getElementsByClassName(this.dataset.affect)[0].getElementsByClassName('options')[0];
		angular.element(currentElement).removeClass("animate");

		track = true;
		hasTouches = !!event.touches;
		startEvent = {x: hasTouches ? event.touches[0].clientX : event.clientX, y: hasTouches ? event.touches[0].clientY : event.clientY};
		currentPosition = currentElement.offsetTop;

		event.preventDefault();
		event.stopPropagation();
	});

	scrollElements.bind(touchmove, function(event){
		if(!track) { return; }
		var newEvent = hasTouches ? event.touches[0] : event;
		var change = newEvent.clientY - startEvent.y;

		newPosition = currentPosition + change;
		if(newPosition > timeSpacing / 2 - 1){
			newPosition = timeSpacing / 2 - 1;
		}
		if(newPosition < -(currentElement.offsetHeight - timeSpacing * numShown + timeSpacing / 2 - 1)){
			newPosition = -(currentElement.offsetHeight - timeSpacing * numShown + timeSpacing / 2 - 1);
		}
		angular.element(currentElement).css({
			"top": newPosition + "px"
		});

		event.preventDefault();
		event.stopPropagation();
	});
	scrollElements.bind(touchend, function(event){
		if(!track){ return; }
		track = false;

		var differenceFromThirty = (-newPosition)  % timeSpacing;

		angular.element(currentElement).addClass("animate");
		differenceFromThirty = differenceFromThirty > timeSpacing / 2 ? differenceFromThirty - timeSpacing : differenceFromThirty;

		angular.element(currentElement).css({
			"top": (newPosition + differenceFromThirty) + "px"
		});
		event.preventDefault();
		event.stopPropagation();
	});

	scrollElements.bind("mousewheel", function(e){
		var delta = e.wheelDelta ? e.wheelDelta : e.detail ? -e.detail : 0;
		currentElement = document.getElementById("st_time").getElementsByClassName(this.dataset.affect)[0].getElementsByClassName('options')[0];

		newPosition = currentElement.offsetTop - (delta > 0 ? timeSpacing : -timeSpacing);
		if(newPosition > 0){
			newPosition = 0;
		}
		if(newPosition < -(currentElement.offsetHeight - timeSpacing * numShown)){
			newPosition = -(currentElement.offsetHeight - timeSpacing * numShown);
		}

		angular.element(currentElement).css({
			top: newPosition + "px"
		});
		event.preventDefault();
		event.stopPropagation();
	});

	$scope.save = function(){
		var hour = -document.getElementById("st_time").getElementsByClassName("hourSelect")[0].getElementsByClassName('options')[0].offsetTop / timeSpacing;
		var minute = -document.getElementById("st_time").getElementsByClassName("minuteSelect")[0].getElementsByClassName('options')[0].offsetTop / timeSpacing * $scope.minuteIncrements;
		st_serviceTime.toggleTime(hour * 60 + minute + 0.1);
	};
}