//'use strict';
var DiscMovement = (function () {
	function DiscMovement(left, right) {
		this.canvasLeft = left;
		this.canvasRight = right;
		this.canvasLeftContext = left.getContext ? left.getContext("2d") : false;
		this.canvasRightContext = right.getContext ? right.getContext("2d") : false;

		this.fieldLength = 120;
		this.endzoneLength = 25;

		this.points = {};
		this.points[fieldSide.left] = [];
		this.points[fieldSide.right] = [];

		this.gradientGood = GetGradient([
			{step:0.45, color:"rgb(255,255,255)"},
			{step:1, color:"rgb(0,0,255)"}
		]);
		this.gradientBad = GetGradient([
			{step:0.45, color:"rgb(255,255,255)"},
			{step:1, color:"rgb(255,0,0)"}
		]);
		this.gradientGoal = GetGradient([
			{step:0.45, color:"rgb(255,255,255)"},
			{step:1, color:"rgb(0,255,0)"}
		]);


		this.lineWidth = 3;
	}
	DiscMovement.prototype.generationTime = 0;
	DiscMovement.prototype.drawFieldLines = function(context){
		var fiveYards = this.canvasLeft.width * 5 / this.fieldLength;
		var endzoneLength = this.canvasLeft.width * this.endzoneLength / this.fieldLength;

		context.lineWidth = 2;
		context.strokeStyle = "#000000";
		context.beginPath();
		context.moveTo(fiveYards, fiveYards);
		context.lineTo(this.canvasLeft.width - fiveYards, fiveYards);
		context.lineTo(this.canvasLeft.width - fiveYards, this.canvasLeft.height - fiveYards);
		context.lineTo(fiveYards, this.canvasLeft.height - fiveYards);
		context.closePath();
		context.moveTo(fiveYards + endzoneLength, fiveYards);
		context.lineTo(fiveYards + endzoneLength, this.canvasLeft.height - fiveYards);
		context.moveTo(this.canvasLeft.width - (fiveYards + endzoneLength), fiveYards);
		context.lineTo(this.canvasLeft.width - (fiveYards + endzoneLength), this.canvasLeft.height - fiveYards);
		context.stroke();
	};
	DiscMovement.prototype.calc = function(points, teamStrategy){
		this.points[fieldSide.left].length = 0;
		this.points[fieldSide.right].length = 0;

		for(var x = 0; x < points.length; x++){
			var point = points[x];
			for(var y = 0; y < point.actions.length - 1; y++){
				var action = point.actions[y];
				var nextAction = point.actions[y + 1];

				if(action.s === teamStrategy && nextAction && action.x && action.y){
					var type = 0;
					switch(nextAction.a){
						case actionTypes.block:
						case actionTypes.throwaway:
						case actionTypes.stallDown:
							type = passType.bad;
							break;
						case actionTypes.point:
							type = passType.goal;
							break;
						case actionTypes.pass:
							type = passType.good;
							break;
					}
					if(type){
						this.points[point.fieldSide].push({
							type: type,
							x0: action.x,
							y0: action.y,
							x1: nextAction.x,
							y1: nextAction.y
						})
					}
				}
			}
		}

		this.updateCanvas();
	};
	DiscMovement.prototype.updateCanvas = function(){
		var start = GetMicroTime();

		this.drawPoints(this.canvasLeftContext, this.points[fieldSide.left]);
		this.drawPoints(this.canvasRightContext, this.points[fieldSide.right]);

		this.generationTime = GetMicroTime() - start;
	};
	DiscMovement.prototype.drawPoints = function(context, points){
		context.clearRect(0, 0, this.canvasLeft.width, this.canvasLeft.height);
		this.drawFieldLines(context);
		context.lineWidth = this.lineWidth;
		context.beginPath();

		for(var x = 0; x < points.length; x++){
			var x0 = Math.round(points[x].x0 / fieldPositionPrecision * this.canvasLeft.width);
			var y0 = Math.round(points[x].y0 / fieldPositionPrecision * this.canvasLeft.height);
			var x1 = Math.round(points[x].x1 / fieldPositionPrecision * this.canvasLeft.width);
			var y1 = Math.round(points[x].y1 / fieldPositionPrecision * this.canvasLeft.height);

			var angle = -Math.atan2(y1 - y0, x1 - x0);
			var arrowHead = ArrowHead(x1, y1, angle, this.lineWidth * 2, Math.PI / 6);

			this.drawLineGradient(context, x0, y0, x1, y1, points[x].type);
			this.drawLineGradient(context, arrowHead.x0, arrowHead.y0, arrowHead.x1, arrowHead.y1, points[x].type);
			this.drawLineGradient(context, arrowHead.x0, arrowHead.y0, arrowHead.x2, arrowHead.y2, points[x].type);
		}
	};
	DiscMovement.prototype.drawLineGradient = function(context, x0, y0, x1, y1, type){
		var slope = - 1 / ((y1 - y0) / (x1 - x0));

		var shiftLeft = Math.sqrt((this.lineWidth * this.lineWidth) / (slope * slope + 1))/2;
		var shiftUp = Math.sqrt((this.lineWidth * this.lineWidth) / (1 + 1 / (slope * slope)))/2;

		if(slope < 0){
			shiftLeft = -shiftLeft;
		}

		var gradient = context.createLinearGradient(x0 + shiftLeft, y0 + shiftUp, x0 - shiftLeft, y0 - shiftUp);
		gradient.addColorStop(0, "rgba(255,255,255,0)");

		var red = type === passType.bad ? 255 : 0;
		var green = type === passType.goal ? 255 : 0;
		var blue = type === passType.good ? 255 : 0;

		gradient.addColorStop(0.5, "rgba(" + red + "," + green + "," + blue + ",1)");
		gradient.addColorStop(1, "rgba(25,255,255,0)");
		context.strokeStyle = gradient;
		context.beginPath();
		context.moveTo(x0,y0);
		context.lineTo(x1,y1);
		context.closePath();
		context.stroke();
	};
	DiscMovement.prototype.reDraw = function(){
		this.updateCanvas();
	};
	return DiscMovement;
})();