'use strict';
var Field = (function(){
	function Field(element){
		this.padding = 0.04;
		this.element = element;
		this.goRight = false;
		this.initValues();
	}
	Field.prototype.drawArrow = function(goRight){
		this.goRight = goRight;
		var arrowHead = ArrowHead(this.goRight ? this.rightX : this.leftX, this.goRight ? this.rightY : this.leftY, this.goRight ? 0 : Math.PI, this.arrowLength, this.arrowAngle);
		this.context.lineWidth = this.padding * this.element.height;
		this.context.beginPath();
		this.context.moveTo(arrowHead.x0, arrowHead.y0);
		this.context.lineTo(arrowHead.x1, arrowHead.y1);
		this.context.moveTo(arrowHead.x0, arrowHead.y0);
		this.context.lineTo(arrowHead.x2, arrowHead.y2);
		this.context.moveTo(this.leftX, this.leftY);
		this.context.lineTo(this.rightX, this.rightY);
		this.context.closePath();
		this.context.stroke();
	};
	Field.prototype.drawAction = function(lastAction, twoActionsAgo){
		this.lastAction = lastAction;
		this.twoActionsAgo = twoActionsAgo;

		if(this.twoActionsAgo && this.lastAction){
			var lastActionX = this.lastAction.x * this.element.width / fieldPositionPrecision;
			var lastActionY = this.lastAction.y * this.element.height / fieldPositionPrecision;

			var twoActionsAgoX = this.twoActionsAgo.x * this.element.width / fieldPositionPrecision;
			var twoActionsAgoY = this.twoActionsAgo.y * this.element.height / fieldPositionPrecision;

			var angle = -Math.atan2(lastActionY - twoActionsAgoY, lastActionX - twoActionsAgoX);

			var arrowHead = ArrowHead(lastActionX, lastActionY, angle, this.arrowLength / 4, this.arrowAngle);
			this.context.lineWidth = this.padding * this.element.height / 3;
			this.context.beginPath();
			this.context.moveTo(arrowHead.x0, arrowHead.y0);
			this.context.lineTo(arrowHead.x1, arrowHead.y1);
			this.context.moveTo(arrowHead.x0, arrowHead.y0);
			this.context.lineTo(arrowHead.x2, arrowHead.y2);
			this.context.moveTo(twoActionsAgoX, twoActionsAgoY);
			this.context.lineTo(lastActionX, lastActionY);
			this.context.closePath();
			this.context.stroke();
		}
	};
	Field.prototype.draw = function(lastAction, twoActionsAgo, goRight){
		this.context.clearRect(0, 0, this.element.width, this.element.height);
		this.drawArrow(goRight);
		this.drawAction(lastAction, twoActionsAgo);
	};
	Field.prototype.initValues = function(){
		this.context = this.element.getContext ? this.element.getContext("2d") : false;
		this.leftX = this.padding * contentWidth * 2;
		this.leftY = this.element.height / 2;
		this.rightX = this.element.width - this.leftX;
		this.rightY = this.leftY;
		this.arrowAngle = Math.PI / 6;
		this.arrowLength = Math.sqrt(this.element.height * this.element.height / 2)/2;

		this.context.lineCap = "round";
		this.context.strokeStyle = 'rgba(0,0,0,0.5)';
	};
	Field.prototype.reDraw = function(){
		this.initValues();
		this.draw(this.lastAction, this.twoActionsAgo, this.goRight);
	};
	return Field;
})();