//'use strict';
var HeatMap = (function () {
	function HeatMap(heatMapCanvasLeft, heatMapCanvasRight) {
		this.heatMapCanvasLeft = heatMapCanvasLeft;
		this.heatMapCanvasLeftContext = this.heatMapCanvasLeft.getContext ? this.heatMapCanvasLeft.getContext("2d") : false;
		this.heatMapCanvasRight = heatMapCanvasRight;
		this.heatMapCanvasRightContext = this.heatMapCanvasRight.getContext ? this.heatMapCanvasRight.getContext("2d") : false;
		this.heatMapOptions = heatMapOptions;
		this.opacity = 180;
		this.dummyCanvas = document.createElement("canvas");

		this.fieldLength = 120;
		this.endzoneLength = 25;

		this.points = {};
		this.points[fieldSide.left] = {};
		this.points[fieldSide.right] = {};

		var gradientCanvas = document.createElement("canvas");
		var gradientCanvasContext = gradientCanvas.getContext("2d");

		gradientCanvas.width = 1;
		gradientCanvas.height = 256;

		var grad = gradientCanvasContext.createLinearGradient(0, 0, 1, 256);
		this.gradient = [
			{step:0.45, color:"rgb(0,0,255)"},
			{step:0.55, color:"rgb(0,255,255)"},
			{step:0.65, color:"rgb(0,255,0)"},
			{step:0.85, color:"rgb(255,255,0)"},
			{step:1.0, color:"rgb(255,0,0)"}
		];
		for (var x = 0, len = this.gradient.length; x < len; x++) {
			grad.addColorStop(this.gradient[x].step, this.gradient[x].color);
		}

		gradientCanvasContext.fillStyle = grad;
		gradientCanvasContext.fillRect(0, 0, 1, 256);

		this.gradient = gradientCanvasContext.getImageData(0, 0, 1, 256).data;

		this.updateDummy();
	}
	HeatMap.prototype.updateDummy = function(){
		if(this.dummyCanvas.width == this.heatMapCanvasLeft.width && this.dummyCanvas.height == this.heatMapCanvasLeft.height){
			return false;
		}

		this.dummyCanvas.width = this.heatMapCanvasLeft.width;
		this.dummyCanvas.height = this.heatMapCanvasLeft.height;
		this.dummyCanvasContext = this.dummyCanvas.getContext("2d");
		this.circleRadius = Math.round(this.heatMapCanvasLeft.width * 0.04);
		return true;
	};
	HeatMap.generationTime = 0;
	HeatMap.prototype.highest = 0;
	HeatMap.prototype.calculate = function (points, teamStrategy) {
		this.highest = 0;
		this.points[fieldSide.left] = {};
		this.points[fieldSide.right] = {};

		var steps = 1.1;

		for(var x = 0; x < points.length; x++){
			var point = points[x];
			for(var y = 0; y < point.actions.length; y++){
				var action = point.actions[y];
				if(action.s === teamStrategy){
					if(this.heatMapOptions.available[action.a] && this.heatMapOptions.available[action.a].value){
						var xPos = Math.round(action.x / fieldPositionPrecision * 100 / steps) * steps;
						var yPos = Math.round(action.y / fieldPositionPrecision * 100 / steps) * steps;

						if(!this.points[point.fieldSide][xPos]){
							this.points[point.fieldSide][xPos] = {};
						}
						if(!this.points[point.fieldSide][xPos][yPos]){
							this.points[point.fieldSide][xPos][yPos] = 0;
						}
						this.points[point.fieldSide][xPos][yPos]++;
						if(this.highest < this.points[point.fieldSide][xPos][yPos]){
							this.highest = this.points[point.fieldSide][xPos][yPos];
						}
					}
				}
			}
		}

		this.updateCanvas();
	};
	HeatMap.prototype.drawFieldLines = function(context){
		var fiveYards = this.heatMapCanvasLeft.width * 5 / this.fieldLength;
		var endzoneLength = this.heatMapCanvasLeft.width * this.endzoneLength / this.fieldLength;

		context.lineWidth = 2;
		context.strokeStyle = "#000000";
		context.beginPath();
		context.moveTo(fiveYards, fiveYards);
		context.lineTo(this.heatMapCanvasLeft.width - fiveYards, fiveYards);
		context.lineTo(this.heatMapCanvasLeft.width - fiveYards, this.heatMapCanvasLeft.height - fiveYards);
		context.lineTo(fiveYards, this.heatMapCanvasLeft.height - fiveYards);
		context.closePath();
		context.moveTo(fiveYards + endzoneLength, fiveYards);
		context.lineTo(fiveYards + endzoneLength, this.heatMapCanvasLeft.height - fiveYards);
		context.moveTo(this.heatMapCanvasLeft.width - (fiveYards + endzoneLength), fiveYards);
		context.lineTo(this.heatMapCanvasLeft.width - (fiveYards + endzoneLength), this.heatMapCanvasLeft.height - fiveYards);
		context.stroke();
	};
	HeatMap.prototype.updateCanvas = function () {

		var start = GetMicroTime();
		this.drawPoints(this.points[fieldSide.left], this.heatMapCanvasLeftContext);
		this.drawPoints(this.points[fieldSide.right], this.heatMapCanvasRightContext);
		this.generationTime = GetMicroTime() - start;
	};
	HeatMap.prototype.drawPoints = function (points, context) {
		context.clearRect(0, 0, this.heatMapCanvasLeft.width, this.heatMapCanvasLeft.height);

		var diameter = this.circleRadius * 2;
		var eighth = this.circleRadius / 8;

		for (var x in points) {
			if (points.hasOwnProperty(x)) {
				for (var y in points[x]) {
					if (points[x].hasOwnProperty(y)) {
						var xPos = x * this.heatMapCanvasLeft.width / 100;
						var yPos = y * this.heatMapCanvasLeft.height / 100;

						var radgrad = context.createRadialGradient(xPos, yPos, eighth, xPos, yPos, this.circleRadius);
						radgrad.addColorStop(0, 'rgba(0,0,0,' + points[x][y] / this.highest + ')');
						radgrad.addColorStop(1, 'rgba(0,0,0,0)');

						context.fillStyle = radgrad;
						context.fillRect(xPos - this.circleRadius, yPos - this.circleRadius, diameter, diameter);
					}
				}
			}
		}

		var width = this.heatMapCanvasLeft.width;
		var height = this.heatMapCanvasLeft.height;
		var heatMap0 = 0;

		var image = context.getImageData(heatMap0, heatMap0, width, height);
		var imageData = image.data;



		for (var i = 3, len = imageData.length; i < len; i += 4) {
			if (!imageData[i]) {
				continue;
			}
			var offset = imageData[i] * 4;

			var opacity = (255 - (imageData[i] < this.opacity) ? imageData[i] : this.opacity) / 255;
			imageData[i - 3] = this.makeWhite(this.gradient[offset], opacity);//)Math.round(this.gradient[offset]) * opacity);
			imageData[i - 2] = this.makeWhite(this.gradient[offset+ 1], opacity);//Math.round(this.gradient[offset + 1]) * opacity);
			imageData[i - 1] = this.makeWhite(this.gradient[offset+ 2], opacity);//Math.round(this.gradient[offset + 2]) * opacity);
			imageData[i] = 255;
		}

		context.putImageData(image, 0, 0);
		this.drawFieldLines(context);
	};
	HeatMap.prototype.makeWhite = function(color, opacity){
		return 255 - (255 - color) * opacity;
	};
	HeatMap.prototype.reDraw = function(){
		if(this.updateDummy()){
			this.updateCanvas();
		}
	};
	return HeatMap;
})();