//'use strict';
var PlayerMovement = (function () {
	function PlayerMovement(left, right) {
		this.canvasLeft = left;
		this.canvasRight = right;
		this.canvasLeftContext = left.getContext ? left.getContext("2d") : false;
		this.canvasRightContext = right.getContext ? right.getContext("2d") : false;

		this.points = {};
		this.points[fieldSide.left] = [];
		this.points[fieldSide.right] = [];

		this.gradientGood = GetGradient([
			{step:0.45, color:"rgb(255,255,255)"},
			{step:1, color:"rgb(0,0,255)"}
		]);
		this.gradientBad = GetGradient([
			{step:0.45, color:"rgb(255,255,255)"},
			{step:1, color:"rgb(255,0,0)"}
		]);
		this.gradientGoal = GetGradient([
			{step:0.45, color:"rgb(255,255,255)"},
			{step:1, color:"rgb(0,255,0)"}
		]);


		this.maxes = {
			left: this.canvasLeft.width * fieldPositionPrecision,
			right: 0,
			top: this.canvasLeft.height * fieldPositionPrecision,
			bottom: 0
		};
		this.lineWidth = 3;
	}
	PlayerMovement.prototype.generationTime = 0;
	PlayerMovement.prototype.calc = function(points, teamStrategy, unselectedPlayers){
		this.unselectedPlayers = [];
		for(var key in unselectedPlayers){
			if(unselectedPlayers.hasOwnProperty(key)){
				if(unselectedPlayers[key] === true){
					this.unselectedPlayers.push(parseInt(key, 10));
				}
			}
		}

		this.points[fieldSide.left].length = 0;
		this.points[fieldSide.right].length = 0;

		this.maxes = {
			left: this.canvasLeft.width * fieldPositionPrecision,
			right: 0,
			top: this.canvasLeft.height * fieldPositionPrecision,
			bottom: 0
		};

		for(var x = 0; x < points.length; x++){
			var point = points[x];
			for(var y = 0; y < point.actions.length - 1; y++){
				var action = point.actions[y];
				var nextAction = point.actions[y + 1];

				if((teamStrategy === strategy.defense || (action.i && this.unselectedPlayers.indexOf(action.i) === -1)) && action.s === teamStrategy && nextAction && action.x && action.y){
					var type = 0;
					switch(nextAction.a){
						case actionTypes.block:
						case actionTypes.throwaway:
							type = passType.bad;
							break;
						case actionTypes.point:
							type = passType.goal;
							break;
						case actionTypes.pass:
							type = passType.good;
							break;
					}

					if(type){
						this.points[point.fieldSide].push({
							type: type,
							x0: action.x,
							y0: action.y,
							x1: nextAction.x,
							y1: nextAction.y
						});

						var deltaX = nextAction.x - action.x;
						var deltay = nextAction.y - action.y;

						if(deltaX < this.maxes.left){
							this.maxes.left = deltaX;
						}
						else if(deltaX > this.maxes.right){
							this.maxes.right = deltaX;
						}
						if(deltay < this.maxes.top){
							this.maxes.top = deltay;
						}
						else if(deltay > this.maxes.bottom){
							this.maxes.bottom = deltay;
						}
					}
				}
			}
		}
		this.totalWidth = this.maxes.right - this.maxes.left;
		this.totalHeight = this.maxes.bottom - this.maxes.top;

		this.stretch = {
			x: 1 / (this.totalWidth / fieldPositionPrecision),
			y: 1 / (this.totalHeight / fieldPositionPrecision)
		};


		this.updateCanvas();
	};
	PlayerMovement.prototype.updateCanvas = function(){
		var start = GetMicroTime();

		this.shift = {
			x: -((this.maxes.left / this.totalWidth) * this.canvasLeft.width),
			y: -((this.maxes.top / this.totalHeight) * this.canvasLeft.height)
		};

		this.drawPoints(this.canvasLeftContext, this.points[fieldSide.left]);
		this.drawPoints(this.canvasRightContext, this.points[fieldSide.right]);

		this.generationTime = GetMicroTime() - start;
	};
	PlayerMovement.prototype.drawPoints = function(context, points){
		context.clearRect(0, 0, this.canvasLeft.width, this.canvasLeft.height);
		context.lineWidth = this.lineWidth;
		context.beginPath();

		for(var x = 0; x < points.length; x++){
			var x0 = Math.floor((points[x].x0 * this.stretch.x) / fieldPositionPrecision * this.canvasLeft.width);
			var y0 = Math.floor((points[x].y0 * this.stretch.y) / fieldPositionPrecision * this.canvasLeft.height);
			var x1 = Math.floor((points[x].x1 * this.stretch.x) / fieldPositionPrecision * this.canvasLeft.width);
			var y1 = Math.floor((points[x].y1 * this.stretch.y) / fieldPositionPrecision * this.canvasLeft.height);

			var leftShift = - x0 + this.shift.x;
			var topShift =  - y0 + this.shift.y;

			x0 = (x0 + leftShift);
			y0 = (y0 + topShift);
			x1 = (x1 + leftShift);
			y1 = (y1 + topShift);

			var angle = -Math.atan2(y1 - y0, x1 - x0);
			var arrowHead = ArrowHead(x1, y1, angle, this.lineWidth * 2, Math.PI / 6);

			this.drawLineGradient(context, x0, y0, x1, y1, points[x].type);
			this.drawLineGradient(context, arrowHead.x0, arrowHead.y0, arrowHead.x1, arrowHead.y1, points[x].type);
			this.drawLineGradient(context, arrowHead.x0, arrowHead.y0, arrowHead.x2, arrowHead.y2, points[x].type);
		}
	};
	PlayerMovement.prototype.drawLineGradient = function(context, x0, y0, x1, y1, type){
		var slope = - 1 / ((y1 - y0) / (x1 - x0));

		var shiftLeft = Math.sqrt((this.lineWidth * this.lineWidth) / (slope * slope + 1))/2;
		var shiftUp = Math.sqrt((this.lineWidth * this.lineWidth) / (1 + 1 / (slope * slope)))/2;

		if(slope < 0){
			shiftLeft = -shiftLeft;
		}

		var gradient = context.createLinearGradient(x0 + shiftLeft, y0 + shiftUp, x0 - shiftLeft, y0 - shiftUp);
		gradient.addColorStop(0, "rgba(255,255,255,0)");

		var red = type === passType.bad ? 255 : 0;
		var green = type === passType.goal ? 255 : 0;
		var blue = type === passType.good ? 255 : 0;

		gradient.addColorStop(0.5, "rgba(" + red + "," + green + "," + blue + ",1)");
		gradient.addColorStop(1, "rgba(25,255,255,0)");
		context.strokeStyle = gradient;
		context.beginPath();
		context.moveTo(x0,y0);
		context.lineTo(x1,y1);
		context.closePath();
		context.stroke();
	};
	PlayerMovement.prototype.reDraw = function(){
		this.updateCanvas();
	};
	return PlayerMovement;
})();