'use strict';
var PointTracker = (function() {
	function PointTracker(game, players, teamName, opponentName, field, ST_Localization, ST_Storage) {
		this.game = game;
		this.points = this.game.points;
		this.players = players;
		this.teamName = teamName;
		this.opponentName = opponentName;
		this.field = field;
		this.ST_Storage = ST_Storage;
		this.ST_Localization = ST_Localization;
		this.update(true);
	}
	PointTracker.prototype.playerAssoc = function(playerId) {
		var playerIndex = ArrayIndexOf(this.players, "id", playerId);
		if(playerIndex !== - 1){ return this.players[playerIndex]; }
		return {};
	};
	PointTracker.prototype.update = function(isInit){
		var pointTrackerScope 			= this;
		this.playerPt 					= {};
		this.scores 					= { scoreFor:0, scoreAgainst:0 };
		this.currentPoint 				= this.points.length !== 0 ? Last(this.points, 1) : false;
		this.lastActionIndex 			= this.currentPoint ? this.currentPoint.actions.length - 1 : -1;
		this.lastAction 				= this.lastActionIndex > -1 ? this.currentPoint.actions[this.lastActionIndex] : false;
		this.showField 					= this.lastAction && this.lastAction.a !== actionTypes.point && this.lastAction.a !== actionTypes.injuryStart && this.lastAction.a !== actionTypes.halfTime;
		this.currentPossession 			= [];
		this.twoActionsAgo 				= this.currentPoint.actions && this.currentPoint.actions.length > 2 ? Last(this.currentPoint.actions, 2) : null;
		this.threeActionsAgo 			= this.currentPoint.actions && this.currentPoint.actions.length > 3 ? Last(this.currentPoint.actions, 3) : null;
		this.hasPassed 					= this.lastAction.a === actionTypes.pass ? this.showField : false;
		this.canPassBack				= ((this.twoActionsAgo && this.twoActionsAgo.a === actionTypes.pass) ? this.showField : false);
		this.hasPickup				 	= this.lastAction.a === actionTypes.pickUp ? this.showField : this.hasPassed;

		this.currentLine 				= this.currentPoint && this.currentPoint.lines ? Last(this.currentPoint.lines, 1).sort(function(a, b){ return pointTrackerScope.playerAssoc(a).type > pointTrackerScope.playerAssoc(b).type; }) : [];
		this.hasHalfTime 				= this.lastAction.a === actionTypes.injuryStart;
		this.arrowDirection 			= this.lastAction.s === strategy.offense ? this.teamName : this.opponentName;
		this.fieldSide					= this.currentPoint ? this.currentPoint.fieldSide === fieldSide.left ? fieldSide.right : fieldSide.left : fieldSide.left;
		this.strategy					= this.currentPoint ? this.lastAction.s === strategy.offense ? strategy.defense : strategy.offense : strategy.offense;

		for(var x = 0, xLen = this.players.length; x < xLen; x++){
			this.playerPt[this.players[x].id] = 0;
		}
		for(var x = 0, xLen = this.points.length; x < xLen; x++){
			if(this.points[x].lines){
				for(var y = 0, yLen = this.points[x].lines.length; y < yLen; y++){
					for(var z = 0, zLen = this.points[x].lines[y].length; z < zLen; z++){
						this.playerPt[this.points[x].lines[y][z]] += 1 / this.points[x].lines.length;
					}
				}
			}
			switch(Last(this.points[x].actions, 1).a){
				case actionTypes.point:
					Last(this.points[x].actions, 1).s === strategy.offense ? this.scores.scoreFor++ : this.scores.scoreAgainst++;
					break;
				case actionTypes.halfTime:
					this.hasHalfTime = true;
					break;
			}
		}
		for(var x = this.lastActionIndex; x >=0 && this.currentPoint.actions[x].s === this.lastAction.s; x--){
			if(this.currentPoint.actions[x].a !== actionTypes.blank){
				this.currentPossession.push(this.currentPoint.actions[x]);
			}
		}
		if(!isInit){
			this.game.modified = GetTime();
			this.ST_Storage.updateStorage();
		}
		this.field.draw(this.lastAction, this.twoActionsAgo, this.currentPoint.fieldSide === fieldSide.left ? this.lastAction.s === strategy.offense : this.lastAction.s === strategy.defense);
		this.currentPossession.reverse();
	};
	PointTracker.prototype.getLastLine = function(strategyType){
		for(var x = this.points.length - 1; x >= 0; x--){
			if((!strategyType || this.points[x].actions[0].s === strategyType) && this.points[x].actions[0].a === actionTypes.blank){
				return Clone(this.points[x].lines[0]);
			}
		}
		return [];
	};
	PointTracker.prototype.nextAction = function () {
		switch (this.lastAction.a) {
			case actionTypes.blank:
				return (this.lastAction.s === strategy.offense ? this.ST_Localization.get("pickup") : this.ST_Localization.get("pull"));
			case actionTypes.pickUp:
			case actionTypes.pass:
				return this.ST_Localization.get("pass");
			default:
				return this.ST_Localization.get("pickup");
		}
	};
	PointTracker.prototype.playerNameFromAction = function (point, actionIndex) {
		var opponent = false;
		if(actionIndex !== 0 && (
			(point.actions[actionIndex].s === strategy.defense && badDefenseActions.indexOf(point.actions[actionIndex].a) !== -1)
			|| (point.actions[actionIndex].s === strategy.offense && badOffenseActions.indexOf(point.actions[actionIndex].a) !== -1)
				)){
			opponent = true;
		}

		var name = "";
		if(point.actions[actionIndex].i){
			name = this.playerAssoc(point.actions[actionIndex].i).name;
			if(opponent){
				name = this.ST_Localization.get("currentMark", [name]);
			}
		}
		else{
			name = (opponent ? this.opponentName : this.teamName);
		}

		return name;
	};
	PointTracker.prototype.actionDescription = function(point, actionIndex) {
		if (!point || !point.actions || !point.actions[actionIndex]) { return ""; }

		var actionPlayerName = "<strong>" + this.playerNameFromAction(point, actionIndex) + "</strong>";
		var lastActionPlayerName = actionIndex > 0 ? "<strong>" + this.playerNameFromAction(point, actionIndex - 1) + "</strong>" : "";

		switch (point.actions[actionIndex].a) {
			case actionTypes.blank:
				return this.ST_Localization.get("startOfThePoint");
			case actionTypes.pickUp:
				return this.ST_Localization.get("pickedUpTheDisc", [actionPlayerName]);
			case actionTypes.pull:
				return this.ST_Localization.get("pulledTheDisc", [actionPlayerName]);
			case actionTypes.pass:
				return this.ST_Localization.get("threwTo", [lastActionPlayerName, actionPlayerName]);
			case actionTypes.drop:
					if(point.actions[actionIndex - 1].a !== actionTypes.pickUp){
						return this.ST_Localization.get("droppedTheDiscFrom", [actionPlayerName, lastActionPlayerName]);
					}
					else{
						return this.ST_Localization.get("droppedTheDisc", [actionPlayerName]);
					}
			case actionTypes.throwaway:
				return this.ST_Localization.get("threwTheDiscAwayTo", [lastActionPlayerName, actionPlayerName]);
			case actionTypes.point:
				return this.ST_Localization.get("threwToPoint", [lastActionPlayerName, actionPlayerName]);
			case actionTypes.timeout:
				return this.ST_Localization.get("calledATimeout", [actionPlayerName]);
			case actionTypes.block:
				return this.ST_Localization.get("gotABlock", [actionPlayerName]);
			case actionTypes.injuryStart:
				return this.ST_Localization.get("anInjuryWasCalled");
			case actionTypes.injuryEnd:
				return this.ST_Localization.get("injuryOver");
			case actionTypes.stallDown:
				return this.ST_Localization.get("gotStalledDown", [actionPlayerName]);
			case actionTypes.halfTime:
				return this.ST_Localization.get("halfTime");
		}
		return "Undefined action type " + point.actions[actionIndex].a;
	};
	PointTracker.prototype.playPoint = function (wind, fieldSide, pointStrategy, line) {
		if (!this.lastAction || this.lastAction.a === actionTypes.point || this.lastAction.a === actionTypes.halfTime) {
			this.points.push({
				wind:wind,
				fieldSide:fieldSide,
				lines:[line],
				actions:[],
				t:GetTime(),
				scoreFor:this.scores.scoreFor,
				scoreAgainst:this.scores.scoreAgainst
			});
			Last(this.points, 1).actions.push({ a:actionTypes.blank, s:pointStrategy, l:1, t: GetTime() - Last(this.points, 1).t});
		}
		else{
			this.currentPoint.lines.push(line);
			this.currentPoint.actions.push({ a:actionTypes.injuryEnd, s:this.lastAction.s, l:this.currentPoint.lines.length - 1 });
		}
		this.update();
	};
	PointTracker.prototype.newAction = function (playerId, xPos, yPos) {
		var actionType = actionTypes.pickUp;
		switch (this.lastAction.a) {
			case actionTypes.blank:
				actionType = this.lastAction.s === strategy.offense ? actionTypes.pickUp : actionTypes.pull;
				break;
			case actionTypes.pickUp:
			case actionTypes.pass:
			case actionTypes.timeout:
				actionType = actionTypes.pass;
		}
		this.currentPoint.actions.push({
			i:parseInt(playerId, 10),
			x:xPos,
			y:yPos,
			a:actionType,
			t:GetTime() - this.currentPoint.t,
			l:this.lastAction.l,
			s:this.lastAction.s
		});
		this.update();
	};
	PointTracker.prototype.updateAction = function (actionType) {
		switch (actionType) {
			case actionTypes.drop:
				if(this.lastAction.a === actionTypes.pickUp){
					this.currentPoint.actions.push(Clone(this.lastAction));
					Last(this.currentPoint.actions, 1).a = actionType;
					Last(this.currentPoint.actions, 1).t = GetTime() - this.currentPoint.t;
					Last(this.currentPoint.actions, 1).s = this.lastAction.s === strategy.offense ? strategy.defense : strategy.offense;
					break;
				}
			case actionTypes.point:
			case actionTypes.throwaway:
			case actionTypes.block:
			case actionTypes.stallDown:
				this.lastAction.a = actionType;
				if (actionType !== actionTypes.point) {
					this.lastAction.s = this.lastAction.s === strategy.offense ? strategy.defense : strategy.offense;
				}
				break;
			case actionTypes.interception:
				this.updateAction(actionTypes.block);
				this.newAction(this.lastAction.i, this.lastAction.x, this.lastAction.y);
				break;
			case actionTypes.callahan:
				this.updateAction(actionTypes.interception);
				this.newAction(this.lastAction.i, this.lastAction.x, this.lastAction.y);
				this.updateAction(actionTypes.point);
				break;
			default:
				this.currentPoint.actions.push(Clone(this.lastAction));
				Last(this.currentPoint.actions, 1).a = actionType;
				Last(this.currentPoint.actions, 1).t = GetTime() - this.currentPoint.t;
				break;
		}
		this.update();
	};
	PointTracker.prototype.passBack = function(){
		if(this.canPassBack && this.threeActionsAgo.i){
			this.updateLastActionPlayer(this.threeActionsAgo.i);
		}
	};
	PointTracker.prototype.updateLastActionPlayer = function(playerId){
		this.lastAction.i = playerId;
		this.update();
	};
	PointTracker.prototype.undo = function () {
		if (this.currentPoint.actions.length === 1) {
			this.points.length--;
		}
		else{
			this.currentPoint.actions.length--;
			this.currentPoint.lines.length = this.lastAction.l;
		}
		this.update();
	};
	PointTracker.prototype.halfTime = function(){
		this.playPoint(this.currentPoint.wind, this.points[0].fieldSide, this.points[0].actions[0].s, []);
		this.lastAction.a = actionTypes.halfTime;
		this.update();
	};
	PointTracker.prototype.debugPoints = function(){
		var temp = "";
		for(var x = 0, xLen = this.points.length; x < xLen; x++){
			temp += "point " + x + "<br />";
			for(var y = 0, yLen = this.points[x].actions.length; y < yLen; y++){
				temp += (this.points[x].actions[y].i && this.playerAssoc(this.points[x].actions[y].i) ? this.playerAssoc(this.points[x].actions[y].i).name : 'Opponent') + " " + actionTypesText[this.points[x].actions[y].a] + " " + strategyText[this.points[x].actions[y].s] + " " + this.points[x].actions[y].t + "<br/>";
			}
		}
		return temp;
	};
	return PointTracker;
})();