'use strict';
var Stats = (function(){
	function Stats(players, yardage){
		this.players = players;

		this.yardage = yardage ? yardage : { horizontal: 120, vertical: 50 };
	}
	Stats.prototype.playerStats = {};
	Stats.prototype.resetPlayers = function(){
		this.playerStats = {};
		for(var x = 0, xLen = this.players.length; x < xLen; x++){
			this.newPlayer(this.players[x]);
		}
		this.newPlayer({id: "unassigned", name: "unassigned"});
	};
	Stats.prototype.newPlayer = function(player){
		this.playerStats[player.id] = {
			name: player.name,
			catches: 0,
			drops: 0,
			throws: 0,
			throwaways: 0,
			goals: 0,
			assists: 0,
			blocks: 0,
			pulls: 0,
			yardageCaught: 0,
			yardageThrown: 0,
			yardageCaughtBackwards: 0,
			yardageThrownBackwards: 0,
			pt: 0
		};
	};
	Stats.prototype.calculateStats = function(points){

		var count = 0;

		this.resetPlayers();
		for(var x = 0, xLen = points.length; x < xLen; x++){
			if(points[x].lines){
				for(var y = 0, yLen = points[x].lines.length; y < yLen; y++){
					for(var z = 0, zLen = points[x].lines[y].length; z < zLen; z++){
						if(this.playerStats[points[x].lines[y][z]]){
							this.playerStats[points[x].lines[y][z]].pt += 1 / points[x].lines.length;
						}
					}
				}
			}
			for(var y = 0, yLen = points[x].actions.length; y < yLen; y++){
				var action = Clone(points[x].actions[y]);
				action.i = action.i ? action.i : "unassigned";
				if(this.playerStats[action.i]){
					var nextAction = ((y+1) < points[x].actions.length ? points[x].actions[y+1] : null);
					var previousAction = ((y-1) >=0 ? points[x].actions[y-1] : null);
					var yardageCaught = ((previousAction && previousAction.x) ? (points[x].fieldSide === fieldSide.left ? (action.x - previousAction.x) / fieldPositionPrecision : (previousAction.x - action.x) / fieldPositionPrecision) : 0) * this.yardage.horizontal / 100;
					var yardageThrown = ((nextAction && nextAction.x) ? (points[x].fieldSide === fieldSide.left ? (nextAction.x - action.x) / fieldPositionPrecision : (action.x - nextAction.x) / fieldPositionPrecision) : 0) * this.yardage.horizontal / 100;

					if(action.s === strategy.offense){
						switch(action.a){
							case actionTypes.point:
								this.playerStats[action.i].goals++;
							case actionTypes.pass:
								this.playerStats[action.i].catches++;
								if(yardageCaught > 0){
									this.playerStats[action.i].yardageCaught += yardageCaught;
								}
								else{
									this.playerStats[action.i].yardageCaughtBackwards += yardageCaught;
								}
								break;
							case actionTypes.block:
								this.playerStats[action.i].blocks++;
								break;
						}
						if(nextAction){
							switch(nextAction.a){
								case actionTypes.block:
									this.playerStats[action.i].throwaways++;
									break;
								case actionTypes.throwaway:
									this.playerStats[action.i].throwaways++;
									break;
								case actionTypes.point:
									this.playerStats[action.i].assists++;
								case actionTypes.pass:
									this.playerStats[action.i].throws++;
									if(yardageThrown > 0){
										this.playerStats[action.i].yardageThrown += yardageThrown;
									}
									else{
										this.playerStats[action.i].yardageThrownBackwards += yardageThrown;
									}
									break;
							}
						}
					}
					else{
						switch(action.a){
							case actionTypes.drop:
								this.playerStats[action.i].drops++;
								break;
							case actionTypes.pull:
								this.playerStats[action.i].pulls++;
								break;
						}
					}
				}
			}
		}
	};
	return Stats;
})();