'use strict';
var Wind = (function () {
	function Wind(windCanvas, $scope, initWind) {
		var windScope = this;
		this.windCanvas = windCanvas;
		this.windCanvasContext = this.windCanvas.getContext ? this.windCanvas.getContext("2d") : false;
		this.$scope = $scope;


		var mouseDown = false;

		angular.element(this.windCanvas).bind("touchstart", function (event) { 	windScope.drawLine(event, event.touches[0]); });
		angular.element(this.windCanvas).bind("touchmove", function (event) { 	windScope.drawLine(event, event.touches[0]); });
		angular.element(this.windCanvas).bind("mousedown", function (event) { 	if (!event.touches) { windScope.drawLine(event, event); mouseDown = true } });
		angular.element(this.windCanvas).bind("mousemove", function (event) { 	if (!event.touches && mouseDown) { windScope.drawLine(event, event); } });
		angular.element(this.windCanvas).bind("mouseup", function () { 			mouseDown = false; });
		this.values = initWind;
		this.reDraw();
	}
	Wind.prototype.values = {intensity:0, direction:0 };
	Wind.prototype.calcWindValue = function(wind, type){
		return Math.floor((type == "x" ? Math.cos(wind.direction) : -Math.sin(wind.direction)) * wind.intensity + this.windCanvas.width / 2);
	};
	Wind.prototype.getOffsets = function(){
		var scrollOffset = document.getElementById("pointScrollContainer") ? document.getElementById("pointScrollContainer").scrollTop : 0;

		return {
			left: GetElementOffset(this.windCanvas, "offsetLeft"),
			top: GetElementOffset(this.windCanvas, "offsetTop") - scrollOffset
		};
	};
	Wind.prototype.drawLine = function(event, points, skipApply){
		if(!this.windCanvasContext){ return; }
		StopEvent(event);


		var offsets = this.getOffsets();

		var xPos = (points.clientX - offsets.left) - this.windCanvas.width / 2;
		var yPos = -(points.clientY - offsets.top - this.windCanvas.width / 2);

		this.values.intensity = Math.round(Math.sqrt(xPos * xPos + yPos * yPos));
		this.values.direction = Math.round(Math.atan2(yPos, xPos) * 100) / 100;

		if (this.values.intensity > this.windCanvas.width / 2) {
			this.values.intensity = this.windCanvas.width / 2;
		}
		var arrowHead = ArrowHead(this.calcWindValue(this.values, "x"), this.calcWindValue(this.values, "y"), this.values.direction, 10, Math.PI / 8);

		this.windCanvasContext.clearRect(0, 0, this.windCanvas.width, this.windCanvas.width);
		this.windCanvasContext.fillStyle = 'black';
		this.windCanvasContext.beginPath();
		this.windCanvasContext.arc(this.windCanvas.width / 2, this.windCanvas.width / 2, 10, 0, Math.PI * 2, true);
		this.windCanvasContext.closePath();
		this.windCanvasContext.fill();
		this.windCanvasContext.beginPath();
		this.windCanvasContext.moveTo(this.windCanvas.width / 2, this.windCanvas.width / 2);
		this.windCanvasContext.lineTo(arrowHead.x0, arrowHead.y0);
		this.windCanvasContext.moveTo(arrowHead.x0, arrowHead.y0);
		this.windCanvasContext.lineTo(arrowHead.x1, arrowHead.y1);
		this.windCanvasContext.lineTo(arrowHead.x2, arrowHead.y2);
		this.windCanvasContext.lineTo(arrowHead.x0, arrowHead.y0);
		this.windCanvasContext.closePath();
		this.windCanvasContext.fill();
		this.windCanvasContext.stroke();

		if (!skipApply) { this.$scope.$apply(); }
	};
	Wind.prototype.reDraw = function(){
		var offsets = this.getOffsets();
		this.windCanvasContext = this.windCanvas.getContext ? this.windCanvas.getContext("2d") : false;
		this.drawLine(null, {clientX:this.calcWindValue(this.values, "x") + offsets.left, clientY:this.calcWindValue(this.values, "y") + offsets.top }, true);
	};
	return Wind;
})();