'use strict';
angular.module('storage', [])
	.service('ST_Storage', function(){
		var storageScope = this;
		var storageData = (window.localStorage.getItem('storageData'));
		//var storageData = dummyData;
		if(!storageData) {
			storageData = {};
		}
		else{
			storageData = JSON.parse(storageData);
		}
		if(!storageData.teams) { 				storageData.teams = []; }
		if(!storageData.games) { 				storageData.games = []; }
		if(!storageData.deleted) { 				storageData.deleted = []; }
		if(!storageData.account) {
			storageData.account = {
				email:'',
				password: ''
			};
		}

		storageScope.addDemoData = function(){
			var temp = GenerateDemoTeamData()
			storageData.teams.push(temp.team);
			return {teamId: temp.teamId, gameId: storageScope.saveGame(temp.teamId, null, GetTime(), {opponentId: temp.opponentId, points: []})};
		};
		storageScope.saveAccount = function(account){
			storageData.account = account;
			storageScope.updateStorage();
		};
		storageScope.getAccount = function(){
			return storageData.account;
		};
		storageScope.getData = function(){
			return storageData;
		};
		storageScope.deleteData = function(){
			storageData = {};
			storageData.teams = [];
			storageData.games = [];
			storageData.deleted = [];
			storageScope.updateStorage();
		};
		storageScope.cleanDeleted = function(){
			if(storageData.deleted)
				storageData.deleted.length = 0;
		};
		storageScope.cleanData = function(){
			var dataToStore = storageScope.getData();
			function RemoveHashKey(data, depth){
				if(!data || (typeof(data) !== "Object" || typeof(data) !== "Array")){ return; }
				if(data["$$hashKey"]){
					delete(data["$$hashKey"]);
				}
				for(var key in data){
					if(data.hasOwnProperty(key)){
						RemoveHashKey(data[key], depth+1);
					}
				}
			}
			RemoveHashKey(dataToStore);
			return dataToStore;
		};
		storageScope.updateStorage = function(){
			window.localStorage.setItem('storageData', JSON.stringify(this.cleanData()));
		};
		storageScope.getSize = function(){
			return unescape(encodeURIComponent(JSON.stringify(window.localStorage))).length;
		};
		/* Team get/set */
		storageScope.getTeamIndex = function(teamId, create){
			var index = ArrayIndexOf(storageData.teams, "id", teamId);
			if(create && index === -1){
				storageData.teams.push({id: teamId });
				index = storageData.teams.length - 1;
			}
			return index;
		};
		storageScope.getTeam = function(teamId){
			var returnData = Clone(storageData.teams[storageScope.getTeamIndex(teamId, false)]);
			returnData.modified = undefined;
			return returnData;
		};
		storageScope.saveTeam = function(teamId, data){
			if(!teamId){ storageData.teams.push({ id: GenerateRandomNumber(), created: GetTime() }); }
			data.synced = data.modified? data.modified : data.synced ? data.synced : 0;
			data.modified = data.modified ? data.modified : GetTime();
			MergeObjects(storageData.teams[teamId ? storageScope.getTeamIndex(teamId, true) : storageData.teams.length - 1], data);
			storageScope.updateStorage();
		};
		storageScope.deleteTeam = function(teamId){
			storageData.deleted.push({key: "teams", id: teamId});
			storageData.teams.splice(storageScope.getTeamIndex(teamId, false), 1);
			storageScope.updateStorage();
		};
		storageScope.getTeamKeyIndex = function(teamId, key, keyId, create){
			if(!keyId){ return {}; }

			var teamIndex = storageScope.getTeamIndex(teamId, false);
			var index = ArrayIndexOf(storageData.teams[teamIndex][key], "id", keyId);

			if(create && index === -1){
				storageData.teams[teamIndex][key].push({id: keyId });
				index = storageData.teams[teamIndex][key].length - 1;
			}

			return index;
		};
		storageScope.getTeamKey = function(teamId, key, keyId){
			if(!keyId){ return {}; }
			var returnData = Clone(storageData.teams[storageScope.getTeamIndex(teamId, false)][key][storageScope.getTeamKeyIndex(teamId, key, keyId, false)]);
			returnData.modified = undefined;
			return returnData;
		};
		storageScope.saveTeamKey = function(teamId, key, keyId, data){
			var teamIndex = storageScope.getTeamIndex(teamId, false);

			if(!storageData.teams[teamIndex][key]){ 	storageData.teams[teamIndex][key] = []; }
			if(!keyId){									storageData.teams[teamIndex][key].push({ id: GenerateRandomNumber(), created:GetTime()}); }
			data.synced = data.modified? data.modified : data.synced ? data.synced : 0;
			data.modified = data.modified ? data.modified : GetTime();
			MergeObjects(storageData.teams[teamIndex][key][keyId ? storageScope.getTeamKeyIndex(teamId, key, keyId, true) : storageData.teams[teamIndex][key].length - 1], data);
			storageScope.updateStorage();
		};
		storageScope.deleteTeamKey = function(teamId, key, keyId){
			if(!storageData.deleted){
				storageData.deleted = [];
			}

			storageData.deleted.push({ key: key, teamId: teamId, id: keyId});
			storageData.teams[storageScope.getTeamIndex(teamId, false)][key].splice(storageScope.getTeamKeyIndex(teamId, key, keyId, false), 1);
			storageScope.updateStorage();
		};
		// Game get/set
		storageScope.getGameIndex = function(gameId, create){
			var index = ArrayIndexOf(storageData.games, "id", gameId);
			if(create && index === -1){
				storageData.games.push({id: gameId });
				index = storageData.games.length - 1;
			}
			return index;
		};
		storageScope.getGame = function(gameId){
			var returnData = storageData.games[storageScope.getGameIndex(gameId, false)];
			returnData.modified = undefined;
			return returnData;
		};
		storageScope.saveGame = function(teamId, tournamentId, gameId, data){
			var gameIndex = gameId ? storageScope.getGameIndex(gameId, true) : storageData.games.length;
			if(!gameId){
				gameId = GenerateRandomNumber();
				storageData.games.push({id: gameId, points: [], created: GetTime() });
			}

			data.synced = (data.modified ? data.modified : (data.synced ? data.synced : 0));
			data.modified = data.modified ? data.modified : GetTime();

			MergeObjects(storageData.games[gameIndex], data);
			storageData.games[gameIndex].teamId = teamId;
			storageData.games[gameIndex].tournamentId = tournamentId;
			storageScope.updateStorage();
			return gameId;
		};
		storageScope.getGames = function(teamId, tournamentId){
			var returnArray = [];
			for(var x = 0, xLen = storageData.games.length; x < xLen; x++){
				if(storageData.games[x].teamId == teamId && ((tournamentId && storageData.games[x].tournamentId == tournamentId) || (!tournamentId && !storageData.games[x].tournamentId))){
					returnArray.push(storageData.games[x]);
				}
			}
			return returnArray;
		};
		storageScope.deleteGame = function(gameId){
			storageData.deleted.push({ key: "games", id: gameId});
			storageData.games.splice(storageScope.getGameIndex(gameId, false), 1);

			storageScope.updateStorage();
		};
		return this;
	});